-------------------------------------------------------------------------------
-- Dr. Kaputa
-- seven segment test bench
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity seven_seg_tb is
end seven_seg_tb;

architecture arch of seven_seg_tb is

component top is
  port (
    clk             : in  std_logic; 
    reset           : in  std_logic;
    a               : in  std_logic_vector(2 downto 0);
    b               : in  std_logic_vector(2 downto 0);
    add             : in  std_logic;
    sub             : in  std_logic;
    output          : out std_logic_vector(6 downto 0)
  );
end component;

signal output       : std_logic_vector(6 downto 0);
constant period     : time := 20ns;                                              
signal clk          : std_logic := '0';
signal reset        : std_logic := '1';
signal a            : std_logic_vector(2 downto 0);
signal b            : std_logic_vector(2 downto 0);
signal add          : std_logic := '0';
signal sub          : std_logic := '0';

begin

-- clock process
clock: process
  begin
    clk <= not clk;
    wait for period/2;
end process; 
 
-- reset process
async_reset: process
  begin
    wait for 2 * period;
    reset <= '0';
    wait;
end process; 

sequential_stimuli: process
begin
  add <= '1';
  a <= (others => '0');
  b <= (others => '0');
  wait for 80 ns;
  for i in 0 to 7 loop
    for j in 0 to 7 loop
      b <= std_logic_vector(unsigned(b) + 1);
      wait for 20 ns;
    end loop;
    a <= std_logic_vector(unsigned(a) + 1);
  end loop;
  add <= '0';
  sub <= '1';
  wait for 20 ns;
  for i in 0 to 8 loop
    a <= std_logic_vector(unsigned(a) + 1);
    for j in 0 to 8 loop
      b <= std_logic_vector(unsigned(b) + 1);
      wait for 20 ns;
    end loop;
  end loop;
end process;

uut: top  
  port map(        
    clk            => clk,
    reset => reset,
    a => a,
    b => b,
    add => add,
    sub => sub,
    output => output
  );
end arch;
