-------------------------------------------------------------------------------
-- Blizzard Finnegan
-- 3-bit input 2 function calculator
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity top is
  port (
    clk             : in  std_logic; 
    reset           : in  std_logic;
    a               : in  std_logic_vector(2 downto 0);
    b               : in  std_logic_vector(2 downto 0);
    add             : in  std_logic;
    sub             : in  std_logic;
    a_disp          : out std_logic_vector(6 downto 0);
    b_disp          : out std_logic_vector(6 downto 0);
    output          : out std_logic_vector(6 downto 0)
  );
end top;

architecture beh of top is

component generic_adder_beh is
  generic (
    bits       : integer := 4
  );
  port (
    a           : in  std_logic_vector(bits-1 downto 0); 
    b           : in  std_logic_vector(bits-1 downto 0);
    cin         : in std_logic;
    sum         : out std_logic_vector(bits-1 downto 0);
    cout        : out std_logic
  );  
end component;  

component seven_seg is
  port (
    clk             : in  std_logic; 
    reset           : in  std_logic;
    bcd             : in std_logic_vector(3 downto 0);
    seven_seg_out   : out std_logic_vector(6 downto 0)
  );
end component;

component double_buffer is
  generic (
    bits : integer := 3
  );
  port(
    d     : in  std_logic_vector(bits - 1 downto 0);
    clk   : in  std_logic;
    reset : in  std_logic;
    q     : out std_logic_vector(bits - 1 downto 0)
  );
end component;

constant INPUT_BITS : integer := 4;
constant OUTPUT_BITS : integer := 7;
signal add_toggle : std_logic := '1';
signal sub_toggle : std_logic;
signal add_sig : std_logic_vector(INPUT_BITS - 1 downto 0);
signal sub_sig : std_logic_vector(INPUT_BITS - 1 downto 0);
signal a_buf : std_logic_vector(INPUT_BITS - 2 downto 0);
signal b_buf : std_logic_vector(INPUT_BITS - 2 downto 0);
signal resized_a : std_logic_vector(INPUT_BITS - 1 downto 0);
signal resized_b : std_logic_vector(INPUT_BITS - 1 downto 0);
signal sub_b : std_logic_vector(INPUT_BITS - 1 downto 0);
signal output_bcd : std_logic_vector(INPUT_BITS - 1 downto 0);
signal output_bcd_sig : std_logic_vector(INPUT_BITS - 1 downto 0);
signal output_sig : std_logic_vector(OUTPUT_BITS - 1 downto 0);
signal b_disp_sig : std_logic_vector(OUTPUT_BITS - 1 downto 0);
signal a_disp_sig : std_logic_vector(OUTPUT_BITS - 1 downto 0);
signal b_disp_sig_buf : std_logic_vector(OUTPUT_BITS - 1 downto 0);
signal a_disp_sig_buf : std_logic_vector(OUTPUT_BITS - 1 downto 0);
signal add_clk : std_logic;
signal sub_clk : std_logic;

begin

resized_a <= '0' & a_buf;
resized_b <= '0' & b_buf;
sub_b <= std_logic_vector(unsigned(not resized_b) + 1);

a_buff : double_buffer
  generic map (
    bits => 3
  )
  port map (
    d => a,
    q => a_buf,
    clk => clk,
    reset => reset
  );
  
b_buff : double_buffer
  generic map (
    bits => 3
  )
  port map (
    d => b,
    q => b_buf,
    clk => clk,
    reset => reset
  );

adder: generic_adder_beh
  generic map (
    bits => INPUT_BITS
  )
  port map (
    a => resized_a,
    b => resized_b,
    cin => '0',
    sum => add_sig,
    cout => open
  );

subtracter: generic_adder_beh
  generic map (
    bits => INPUT_BITS
  )
  port map (
    a => resized_a,
    b => sub_b,
    cin => '0',
    sum => sub_sig,
    cout => open
  );

output_display: seven_seg
  port map (
    clk => clk,
    reset => reset,
    bcd => output_bcd,
    seven_seg_out => output_sig
  );

a_display: seven_seg
  port map (
    clk => clk,
    reset => reset,
    bcd => resized_a,
    seven_seg_out => a_disp_sig
  );
b_display: seven_seg
  port map (
    clk => clk,
    reset => reset,
    bcd => resized_b,
    seven_seg_out => b_disp_sig
  );

a_disp_buff_one: double_buffer
  generic map(
    bits => 7
  )
  port map(
    d => a_disp_sig,
    q => a_disp,
    clk => clk,
    reset => reset
  );

b_disp_buff_one: double_buffer
  generic map(
    bits => 7
  )
  port map(
    d => b_disp_sig,
    q => b_disp,
    clk => clk,
    reset => reset
  );

output_buff: double_buffer
  generic map(
    bits => 7
  )
  port map(
    d => output_sig,
    q => output,
    clk => clk,
    reset => reset
  );

-- SR FF
add_clk <= add and clk;
sub_clk <= sub and clk;
add_toggle <= sub_toggle nor add_clk;
sub_toggle <= add_toggle nor sub_clk;

mux: process(add_toggle,sub_toggle)
begin
  if (add_toggle = '1') then
    output_bcd <= add_sig;
  elsif (sub_toggle = '1') then
    output_bcd <= sub_sig;
  end if;
end process;

end beh;
