-------------------------------------------------------------------------------
-- Blizzard Finnegan
-- Finite State Machine
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity fsm is
  port(
    clk       : in  std_logic;
    reset     : in  std_logic;
    save      : in  std_logic;
    read      : in  std_logic;
    exec      : in  std_logic;
    loopback  : in  std_logic_vector(7 downto 0);
    result    : in  std_logic_vector(7 downto 0);
    we        : out std_logic;
    addr      : out std_logic_vector(1 downto 0);
    reg_input : out  std_logic_vector(7 downto 0)
  );
end entity fsm;

architecture behaviour of fsm is

type state is (pause,save_val,read_1,read_2,execute);

component double_buffer is
  generic(
    bits : integer := 8
  );
  port(
    d     : in  std_logic_vector(bits - 1 downto 0);
    clk   : in  std_logic;
    reset : in  std_logic;
    q     : out std_logic_vector(bits - 1 downto 0)
  );
end component;

signal address: std_logic_vector(1 downto 0) := (others => '0');
signal value_select : std_logic := '0';
signal state_store: state;
signal write_enable : std_logic := '0';
signal buff_loop : std_logic_vector(7 downto 0);

begin

addr <= address;
we <= write_enable;

-- loop_buffer : process(clk,reset) is
-- begin
--   if(reset = '1') then
--     buff_loop <= (others => '0');
--   elsif (rising_edge(clk)) then
--     buff_loop <= loopback;
--   end if;
-- end process;

output_mux : process(state_store,value_select) is
begin
  if(value_select = '1') then
    reg_input <= loopback;
  else
    reg_input <= result;
  end if;
end process;

process(clk,reset) is
begin
  if (reset = '1') then
    state_store <= pause;
    write_enable <= '0';
    value_select <= '0';
    address <= "00";
  elsif (rising_edge(clk)) then
    case state_store is
      when pause =>
        write_enable <= '0';
        address <= "00";
        value_select <= '0';
        if (exec = '1') then
          state_store <= execute;
        elsif (save = '1') then
          state_store <= save_val;
        elsif (read = '1') then
          state_store <= read_1;
        else
          state_store <= pause;
        end if;
      when save_val =>
        write_enable <= '1';
        address <= "01";
        value_select <= '1';
        state_store <= pause;
      when read_1 =>
        write_enable <= '0';
        address <= "01";
        value_select <= '0';
        state_store <= read_2;
      when read_2 =>
        write_enable <= '1';
        address <= "00";
        value_select <= '1';
        state_store <= pause;
      when execute =>
        write_enable <= '1';
        value_select <= '0';
        address <= "00";
        state_store <= pause;
    end case;
  end if;
end process;

end behaviour;
