-------------------------------------------------------------------------------
-- Blizzard Finnegan
-- Finite State Machine
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity fsm is
  port(
    clk       : in  std_logic;
    reset     : in  std_logic;
    exec      : in  std_logic;
    op_in     : in  std_logic_vector(3 downto 0);
    loopback  : in  std_logic_vector(7 downto 0);
    result    : in  std_logic_vector(7 downto 0);
    we        : out std_logic;
    op_out    : out std_logic_vector(1 downto 0);
    ram_addr  : out std_logic_vector(1 downto 0);
    ram_input : out std_logic_vector(7 downto 0);
    rom_addr  : out std_logic_vector(4 downto 0)
  );
end entity fsm;

architecture behaviour of fsm is

type state is (pause,save_val,read_1,read_2,add,sub,mult,div,math_2);
signal ram_address: std_logic_vector(1 downto 0) := (others => '0');
signal value_select : std_logic := '0';
signal state_store: state := pause;
signal write_enable : std_logic := '0';
signal buff_loop : std_logic_vector(7 downto 0);
signal rom_address : std_logic_vector(4 downto 0) := (others => '0');

begin

ram_addr <= ram_address;
rom_addr <= rom_address;
we <= write_enable;



output_mux : process(clk) is
begin
  if(value_select = '1') then
    ram_input <= loopback;
  else
    ram_input <= result;
  end if;
end process;

alu_op : process(state_store) is
begin
  case op_in is
    when "0000" =>
      op_out <= "00";
    when "0001" =>
      op_out <= "01";
    when "0011" =>
      op_out <= "10";
    when "0010" =>
      op_out <= "11";
    when others =>
      op_out <= "00";
  end case;
end process;

state_proc: process(clk,reset) is
begin
  if (reset = '1') then
    state_store <= pause;
    write_enable <= '0';
    value_select <= '0';
    ram_address <= "00";
    rom_address <= (others => '0');
  elsif (rising_edge(clk)) then
    case state_store is
      when pause =>
        write_enable <= '0';
        ram_address <= "00";
        value_select <= '0';
        if (exec = '1') then
          case op_in is
            when "0000" =>
              state_store <= add;
            when "0001" =>
              state_store <= sub;
            when "0011" =>
              state_store <= mult;
            when "0010" =>
              state_store <= div;
            when "0110" =>
              state_store <= save_val;
            when "0111" =>
              state_store <= read_1;
            when others => 
              state_store <= pause;
          end case;
      end if;
      when save_val =>
        write_enable <= '1';
        ram_address <= "01";
        value_select <= '1';
        state_store <= pause;
        rom_address <= std_logic_vector((unsigned(rom_address) + 1) );
      when read_1 =>
        write_enable <= '0';
        ram_address <= "01";
        value_select <= '0';
        state_store <= read_2;
      when read_2 =>
        write_enable <= '1';
        ram_address <= "00";
        value_select <= '1';
        state_store <= pause;
        rom_address <= std_logic_vector((unsigned(rom_address) + 1) );
      when mult | add | sub | div =>
        write_enable <= '0';
        value_select <= '0';
        ram_address <= "00";
        state_store <= math_2;
      when math_2 => 
        write_enable <= '1';
        value_select <= '0';
        ram_address <= "00";
        state_store <= pause;
        rom_address <= std_logic_vector((unsigned(rom_address) + 1) );
    end case;
  end if;

end process;

end behaviour;
