-- Dr. Kaputa
-- Lab 8: DJ Roomba 3000 

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity dj_roomba_3000 is 
  port(
    clk                 : in std_logic;
    reset               : in std_logic;
    execute_btn         : in std_logic;
    sync                : in std_logic;
    led                 : out std_logic_vector(7 downto 0);
    audio_out           : out std_logic_vector(15 downto 0)
  );
end dj_roomba_3000;

architecture beh of dj_roomba_3000 is
  -- instruction memory
  component rom_instructions
    port(
      address    : in std_logic_vector (4 DOWNTO 0);
      clock      : in std_logic  := '1';
      q          : out std_logic_vector (7 DOWNTO 0)
    );
  end component;
  
  -- data memory
  component rom_data
    port(
      address  : in std_logic_vector (13 DOWNTO 0);
      clock    : in std_logic  := '1';
      q        : out std_logic_vector (15 DOWNTO 0)
    );
  end component;
type state is (idle,fetch,decode,execute,decode_error);
signal state_store: state := idle;

signal data_address  : std_logic_vector(13 downto 0);
signal addr : std_logic_vector(4 DOWNTO 0) := ( others => '0' );
signal instruction : std_logic_vector(7 DOWNTO 0);
signal op_code : std_logic_vector(1 DOWNTO 0);
signal repeat : std_logic;
signal seek_addr : std_logic_vector(4 DOWNTO 0);
signal exec_signal : std_logic;

signal exec_z, exec_zz, exec_zzz : std_logic;

begin

-- data instantiation
u_rom_data_inst : rom_data
  port map (
    address    => data_address,
    clock      => clk,
    q          => audio_out
  );

u_rom_instruct : rom_instructions
  port map(
    address => addr,
    clock => clk,
    q => instruction
  );

  led <= instruction;


  process(clk,reset)
  begin
    if (reset = '1') then
      exec_z <= '0';
      exec_zz <= '0';
      exec_zzz <= '0';
      exec_signal <= '0';
    elsif (rising_edge(clk)) then
      exec_z <= execute_btn;
      exec_zz <= exec_z;
      exec_zzz <= exec_zz;
      exec_signal <= exec_zzz and (not exec_zz);
    end if;
  end process;

  -- Synchroniser process
  process(clk,reset)
  begin
    if (reset = '1') then
      addr <= (others => '0');
      state_store <= idle;
    elsif (rising_edge(clk)) then
      case state_store is
	when fetch => 
	  state_store <= decode;
	when decode => 
	  op_code <= instruction(7 DOWNTO 6);
	  repeat <= instruction(5);
	  seek_addr <= instruction(4 DOWNTO 0);
	  state_store <= execute;
	when execute =>
	  if (exec_signal = '1') then
	    addr <= std_logic_vector( unsigned(addr) + 1);
	    state_store <= fetch;
	  end if;
	when decode_error => 
	  state_store <= fetch;
	when others =>
      end case;
    end if;
  end process;

  -- State machine logic process
  -- 4 states: idle, fetch, decode, execute, decode-error
  -- case statement
  -- idle: when reset=0, -> execute; reset PC to 0
  -- fetch: get inst from ROM; increment PC by 1
  -- Decode: case statement on highest 3 bits, else goto decode-error
  -- decode error: idle
  -- execute: use data from fetch when sync == 1

  -- LED process(?)
    
  -- data logic process
  process(clk,reset)
  begin 
    if (reset = '1') then 
      data_address <= (others => '0');
    elsif (clk'event and clk = '1') then
      if (sync = '1') then    
	 -- for exec state, case statement against opcodes 
	if ( state_store = execute ) then
	  case op_code is
	    when "00" =>
	      -- Play w/ repeat
	      if (repeat = '1') then
		data_address <= std_logic_vector(unsigned(data_address) + 1 );

	      -- Play, no repeat
	      elsif not(data_address = "11111111111111" ) then
		data_address <= std_logic_vector(unsigned(data_address) + 1 );
	      end if;
	    -- Pause
	    when "01" =>
	      data_address <= data_address;

	    -- Seek
	    when "10" =>
	      data_address <= seek_addr & "000000000";

	    -- Stop
	    when "11" =>
	      data_address <= ( others => '0' );

	    when others =>
	  end case;
	end if;
      end if;
    end if;
  end process;

end beh;
